@@ -1,22 +1,48 @@
-use protobuf::descriptor::*;
-use protobuf::descriptorx::*;
-
 use super::code_writer::*;
 use super::customize::customize_from_rustproto_for_message;
 use super::customize::Customize;
 use super::enums::*;
 use super::field::*;
 use super::rust_types_values::*;
+use file_descriptor::file_descriptor_proto_expr;
 use inside::protobuf_crate_path;
 use oneof::OneofGen;
 use oneof::OneofVariantGen;
+use protobuf::descriptor::*;
+use rust_name::RustIdentWithPath;
+use scope::MessageWithScope;
+use scope::RootScope;
+use scope::WithScope;
 use serde;
 
+use std::fmt;
+
+/// Protobuf message Rust type name
+#[derive(Debug, Clone, PartialEq, Eq)]
+pub(crate) struct RustTypeMessage(pub RustIdentWithPath);
+
+impl fmt::Display for RustTypeMessage {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        fmt::Display::fmt(&self.0, f)
+    }
+}
+
+impl RustTypeMessage {
+    /// Code which emits default instance.
+    pub fn default_instance(&self, customize: &Customize) -> String {
+        format!(
+            "<{} as {}::Message>::default_instance()",
+            self.0,
+            protobuf_crate_path(customize)
+        )
+    }
+}
+
 /// Message info for codegen
-pub struct MessageGen<'a> {
-    pub(crate) message: &'a MessageWithScope<'a>,
-    pub(crate) root_scope: &'a RootScope<'a>,
-    type_name: String,
+pub(crate) struct MessageGen<'a> {
+    pub message: &'a MessageWithScope<'a>,
+    pub root_scope: &'a RootScope<'a>,
+    type_name: RustIdentWithPath,
     pub fields: Vec<FieldGen<'a>>,
     pub lite_runtime: bool,
     customize: Customize,
@@ -46,10 +72,10 @@ impl<'a> MessageGen<'a> {
                 == FileOptions_OptimizeMode::LITE_RUNTIME
         });
         MessageGen {
-            message: message,
-            root_scope: root_scope,
-            type_name: message.rust_name(),
-            fields: fields,
+            message,
+            root_scope,
+            type_name: message.rust_name().into(),
+            fields,
             lite_runtime,
             customize,
         }
@@ -109,7 +135,7 @@ impl<'a> MessageGen<'a> {
         for oneof in self.oneofs() {
             w.if_let_stmt(
                 "::std::option::Option::Some(ref v)",
-                &format!("self.{}", oneof.name())[..],
+                &format!("self.{}", oneof.oneof.field_name())[..],
                 |w| {
                     w.match_block("v", |w| {
                         for variant in oneof.variants_except_group() {
@@ -160,8 +186,9 @@ impl<'a> MessageGen<'a> {
             |w| {
                 w.lazy_static_decl_get_simple(
                     "instance",
-                    &self.type_name,
+                    &self.type_name.to_string(),
                     &format!("{}::new", self.type_name),
+                    &self.customize,
                 );
             },
         );
@@ -203,7 +230,7 @@ impl<'a> MessageGen<'a> {
     }
 
     fn write_impl_self(&self, w: &mut CodeWriter) {
-        w.impl_self_block(&self.type_name, |w| {
+        w.impl_self_block(&self.type_name.to_string(), |w| {
             // TODO: new should probably be a part of Message trait
             w.pub_fn(&format!("new() -> {}", self.type_name), |w| {
                 w.write_line("::std::default::Default::default()");
@@ -215,14 +242,20 @@ impl<'a> MessageGen<'a> {
 
     fn write_unknown_fields(&self, w: &mut CodeWriter) {
         w.def_fn(
-            "get_unknown_fields(&self) -> &::protobuf::UnknownFields",
+            &format!(
+                "get_unknown_fields(&self) -> &{}::UnknownFields",
+                protobuf_crate_path(&self.customize)
+            ),
             |w| {
                 w.write_line("&self.unknown_fields");
             },
         );
         w.write_line("");
         w.def_fn(
-            "mut_unknown_fields(&mut self) -> &mut ::protobuf::UnknownFields",
+            &format!(
+                "mut_unknown_fields(&mut self) -> &mut {}::UnknownFields",
+                protobuf_crate_path(&self.customize)
+            ),
             |w| {
                 w.write_line("&mut self.unknown_fields");
             },
@@ -257,8 +290,9 @@ impl<'a> MessageGen<'a> {
     fn write_descriptor_field(&self, fields_var: &str, field: &FieldGen, w: &mut CodeWriter) {
         let accessor_fn = field.accessor_fn();
         w.write_line(&format!(
-            "{}.push(::protobuf::reflect::accessor::{}(",
+            "{}.push({}::reflect::accessor::{}(",
             fields_var,
+            protobuf_crate_path(&self.customize),
             accessor_fn.sig()
         ));
         w.indented(|w| {
@@ -285,11 +319,18 @@ impl<'a> MessageGen<'a> {
 
     fn write_descriptor_static(&self, w: &mut CodeWriter) {
         w.def_fn(
-            &format!("descriptor_static() -> &'static ::protobuf::reflect::MessageDescriptor"),
+            &format!(
+                "descriptor_static() -> &'static {}::reflect::MessageDescriptor",
+                protobuf_crate_path(&self.customize)
+            ),
             |w| {
                 w.lazy_static_decl_get(
                     "descriptor",
-                    "::protobuf::reflect::MessageDescriptor",
+                    &format!(
+                        "{}::reflect::MessageDescriptor",
+                        protobuf_crate_path(&self.customize)
+                    ),
+                    &self.customize,
                     |w| {
                         let fields = self.fields_except_group();
                         if fields.is_empty() {
@@ -298,16 +339,17 @@ impl<'a> MessageGen<'a> {
                             w.write_line(&format!("let mut fields = ::std::vec::Vec::new();"));
                         }
                         for field in fields {
-                            self.write_descriptor_field("fields", field, w);;
+                            self.write_descriptor_field("fields", field, w);
                         }
                         w.write_line(&format!(
-                            "::protobuf::reflect::MessageDescriptor::new::<{}>(",
+                            "{}::reflect::MessageDescriptor::new_pb_name::<{}>(",
+                            protobuf_crate_path(&self.customize),
                             self.type_name
                         ));
                         w.indented(|w| {
-                            w.write_line(&format!("\"{}\",", self.type_name));
+                            w.write_line(&format!("\"{}\",", self.message.name_to_package()));
                             w.write_line("fields,");
-                            w.write_line("file_descriptor_proto()");
+                            w.write_line(&file_descriptor_proto_expr(&self.message.scope));
                         });
                         w.write_line(")");
                     },
@@ -346,7 +388,9 @@ impl<'a> MessageGen<'a> {
     }
 
     fn write_impl_message(&self, w: &mut CodeWriter) {
-        w.impl_for_block("::protobuf::Message", &self.type_name, |w| {
+        w.impl_for_block(
+            &format!("{}::Message", protobuf_crate_path(&self.customize)),
+            &self.type_name.to_string(), |w| {
             self.write_is_initialized(w);
             w.write_line("");
             self.write_merge_from(w);
@@ -366,14 +410,14 @@ impl<'a> MessageGen<'a> {
                 w.write_line("self as &mut dyn (::std::any::Any)");
             });
             w.def_fn(
-                "into_any(self: Box<Self>) -> ::std::boxed::Box<dyn (::std::any::Any)>",
+                "into_any(self: ::std::boxed::Box<Self>) -> ::std::boxed::Box<dyn (::std::any::Any)>",
                 |w| {
                     w.write_line("self");
                 },
             );
             w.write_line("");
             w.def_fn(
-                "descriptor(&self) -> &'static ::protobuf::reflect::MessageDescriptor",
+                &format!("descriptor(&self) -> &'static {}::reflect::MessageDescriptor", protobuf_crate_path(&self.customize)),
                 |w| {
                     w.write_line("Self::descriptor_static()");
                 },
@@ -392,34 +436,56 @@ impl<'a> MessageGen<'a> {
     }
 
     fn write_impl_value(&self, w: &mut CodeWriter) {
-        w.impl_for_block("::protobuf::reflect::ProtobufValue", &self.type_name, |w| {
-            w.def_fn(
-                "as_ref(&self) -> ::protobuf::reflect::ProtobufValueRef",
-                |w| w.write_line("::protobuf::reflect::ProtobufValueRef::Message(self)"),
-            )
-        })
+        w.impl_for_block(
+            &format!(
+                "{}::reflect::ProtobufValue",
+                protobuf_crate_path(&self.customize)
+            ),
+            &self.type_name.to_string(),
+            |w| {
+                w.def_fn(
+                    &format!(
+                        "as_ref(&self) -> {}::reflect::ReflectValueRef",
+                        protobuf_crate_path(&self.customize)
+                    ),
+                    |w| {
+                        w.write_line(&format!(
+                            "{}::reflect::ReflectValueRef::Message(self)",
+                            protobuf_crate_path(&self.customize)
+                        ))
+                    },
+                )
+            },
+        )
     }
 
     fn write_impl_show(&self, w: &mut CodeWriter) {
-        w.impl_for_block("::std::fmt::Debug", &self.type_name, |w| {
+        w.impl_for_block("::std::fmt::Debug", &self.type_name.to_string(), |w| {
             w.def_fn(
                 "fmt(&self, f: &mut ::std::fmt::Formatter<'_>) -> ::std::fmt::Result",
                 |w| {
-                    w.write_line("::protobuf::text_format::fmt(self, f)");
+                    w.write_line(&format!(
+                        "{}::text_format::fmt(self, f)",
+                        protobuf_crate_path(&self.customize)
+                    ));
                 },
             );
         });
     }
 
     fn write_impl_clear(&self, w: &mut CodeWriter) {
-        w.impl_for_block("::protobuf::Clear", &self.type_name, |w| {
-            w.def_fn("clear(&mut self)", |w| {
-                for f in self.fields_except_group() {
-                    f.write_clear(w);
-                }
-                w.write_line("self.unknown_fields.clear();");
-            });
-        });
+        w.impl_for_block(
+            &format!("{}::Clear", protobuf_crate_path(&self.customize)),
+            &format!("{}", self.type_name),
+            |w| {
+                w.def_fn("clear(&mut self)", |w| {
+                    for f in self.fields_except_group() {
+                        f.write_clear(w);
+                    }
+                    w.write_line("self.unknown_fields.clear();");
+                });
+            },
+        );
     }
 
     #[allow(dead_code)]
@@ -435,8 +501,12 @@ impl<'a> MessageGen<'a> {
             derive.push("Debug");
         }
         w.derive(&derive);
-        serde::write_serde_attr(w, &self.customize, "derive(Serialize, Deserialize)");
-        w.pub_struct(&self.type_name, |w| {
+        serde::write_serde_attr(
+            w,
+            &self.customize,
+            "derive(::serde::Serialize, ::serde::Deserialize)",
+        );
+        w.pub_struct(&self.type_name.to_string(), |w| {
             if !self.fields_except_oneof().is_empty() {
                 w.comment("message fields");
                 for field in self.fields_except_oneof() {
@@ -460,7 +530,7 @@ impl<'a> MessageGen<'a> {
                         };
                         w.field_decl_vis(
                             vis,
-                            &field.rust_name,
+                            &field.rust_name.get(),
                             &field.full_storage_type().to_code(&self.customize),
                         );
                     }
@@ -475,7 +545,7 @@ impl<'a> MessageGen<'a> {
                     };
                     w.field_decl_vis(
                         vis,
-                        oneof.name(),
+                        oneof.oneof.field_name().get(),
                         &oneof.full_storage_type().to_code(&self.customize),
                     );
                 }
